package convert

import (
	"image"
	"io"
	"mime"
	"os"
	"strings"

	"git.perx.ru/perxis/perxis-go/pkg/errors"
)

type (
	EncodeFunc func(w io.Writer, img image.Image) error
	Format     string
)

func (f Format) String() string {
	return string(f)
}

var (
	defaultFormatEncoderRegistry = make(map[Format]EncodeFunc)
	formatExtensions             = make(map[string]Format)
)

func RegisterFormatEncoder(format Format, fn EncodeFunc, extensions ...string) {
	defaultFormatEncoderRegistry[format] = fn
	formatExtensions[format.String()] = format
	for _, ext := range extensions {
		formatExtensions[strings.TrimPrefix(strings.ToLower(ext), ".")] = format
	}
}

func Encode(w io.Writer, format Format, img image.Image) error {
	encoder, ok := defaultFormatEncoderRegistry[format]
	if !ok {
		return errors.Errorf("unknown format: %s", format)
	}
	err := encoder(w, img)
	if err != nil {
		return errors.Wrap(err, "encode image")
	}
	return nil
}

func Decode(r io.Reader) (image.Image, string, error) {
	img, ext, err := image.Decode(r)
	if err != nil {
		return nil, "", errors.Wrap(err, "decode image")
	}
	return img, ext, nil
}

func Open(filename string) (image.Image, string, error) {
	file, err := os.Open(filename)
	if err != nil {
		return nil, "", errors.Wrap(err, "open file")
	}
	defer file.Close()
	img, ext, err := Decode(file)
	if err != nil {
		return nil, "", errors.Wrap(err, "decode file")
	}
	return img, ext, nil
}

func FormatFromExtension(ext string) (Format, error) {
	ext = strings.TrimPrefix(strings.ToLower(ext), ".")
	if f, ok := formatExtensions[ext]; ok {
		return f, nil
	}
	return "", errors.Errorf("unsupported format: %s", ext)
}

func FormatFromMIMEType(typ string) (Format, error) {
	extensions, err := mime.ExtensionsByType(typ)
	if err != nil {
		return "", errors.Wrap(err, "get extensions by mime type")
	}
	if len(extensions) == 0 {
		return "", errors.Errorf("unknown mime type: %s", typ)
	}
	for _, extension := range extensions {
		var format Format
		format, err = FormatFromExtension(extension)
		if err == nil {
			return format, nil
		}
	}
	return "", err
}
