package logs

import (
	"testing"
	"time"

	"git.perx.ru/perxis/perxis-go/id"
	pb "git.perx.ru/perxis/perxis-go/proto/logs"
	"github.com/stretchr/testify/assert"
)

func TestEntry_ToMap(t *testing.T) {
	type fields struct {
		ID        string
		Timestamp time.Time
		LogLevel  Level
		Message   string
		Category  string
		Component string
		Event     string
		ObjectId  *id.ObjectId
		CallerId  *id.ObjectId
		Attr      interface{}
		Tags      []string
	}
	tests := []struct {
		name   string
		fields fields
		want   map[string]interface{}
	}{
		{
			"#1",
			fields{
				"1",
				time.Time{},
				0,
				"message",
				"",
				"",
				"",
				id.MustObjectId("/spaces/<space_id>/envs/<env_id>"),
				id.MustObjectId("/users/<user_id>"),
				nil,
				nil,
			},
			map[string]interface{}{"caller": map[string]interface{}{"type": "user", "user_id": "<user_id>"}, "caller_id": "/users/<user_id>", "category": "", "component": "", "event": "", "id": "1", "level": Level(0), "message": "message", "object": map[string]interface{}{"env_id": "<env_id>", "space_id": "<space_id>", "type": "environment"}, "object_id": "/spaces/<space_id>/envs/<env_id>", "timestamp": time.Date(1, time.January, 1, 0, 0, 0, 0, time.UTC)},
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			e := &Entry{
				ID:        tt.fields.ID,
				Timestamp: tt.fields.Timestamp,
				Level:     tt.fields.LogLevel,
				Message:   tt.fields.Message,
				Category:  tt.fields.Category,
				Component: tt.fields.Component,
				Event:     tt.fields.Event,
				ObjectID:  tt.fields.ObjectId,
				CallerID:  tt.fields.CallerId,
				Attr:      tt.fields.Attr,
				Tags:      tt.fields.Tags,
			}
			assert.Equal(t, tt.want, e.ToMap())
		})
	}
}

func TestFindRequest(t *testing.T) {
	t.Run("From PB", func(t *testing.T) {
		res := FindRequestFromPB(&pb.FindRequest{
			Filter:  nil,
			Options: &pb.FindOptions{},
		})
		assert.True(t, res.Options.After.IsZero())
		assert.True(t, res.Options.Before.IsZero())
	})
	t.Run("To PB", func(t *testing.T) {
		res := FindRequestToPB(&FindRequest{
			Filter:  nil,
			Options: &FindOptions{},
		})
		assert.False(t, res.Options.After.IsValid())
		assert.False(t, res.Options.Before.IsValid())
	})
}

func TestFindResult(t *testing.T) {
	t.Run("From PB", func(t *testing.T) {
		res := FindResultFromPB(&pb.FindResult{
			Options: &pb.FindOptions{Limit: 2, Sort: []string{"timestamp"}, Fields: []string{"level"}},
		})
		assert.True(t, res.Options.After.IsZero())
		assert.True(t, res.Options.Before.IsZero())
		assert.Equal(t, 2, res.Options.Limit)
		assert.Equal(t, []string{"timestamp"}, res.Options.Sort)
		assert.Equal(t, []string{"level"}, res.Options.Fields)
	})
	t.Run("To PB", func(t *testing.T) {
		res := FindResultToPB(&FindResult{
			Options: &FindOptions{Limit: 2, Sort: []string{"timestamp"}, Fields: []string{"level"}},
		})
		assert.False(t, res.Options.After.IsValid())
		assert.False(t, res.Options.Before.IsValid())
		assert.Equal(t, int32(2), res.Options.Limit)
		assert.Equal(t, []string{"timestamp"}, res.Options.Sort)
		assert.Equal(t, []string{"level"}, res.Options.Fields)
	})
	t.Run("From PB: with nil filter and options", func(t *testing.T) {
		res := FindResultFromPB(&pb.FindResult{Filter: nil, Options: nil})
		assert.NotNil(t, res)
		assert.Nil(t, res.Options)
		assert.Nil(t, res.Filter)
	})
	t.Run("To PB: with nil filter and options", func(t *testing.T) {
		res := FindResultToPB(&FindResult{Filter: nil, Options: nil})
		assert.NotNil(t, res)
		assert.Nil(t, res.Options)
		assert.Nil(t, res.Filter)
	})
	t.Run("From PB: nil", func(t *testing.T) {
		res := FindResultFromPB(nil)
		assert.Nil(t, res)
	})
	t.Run("To PB: nil", func(t *testing.T) {
		res := FindResultToPB(nil)
		assert.Nil(t, res)
	})
}
