package middleware

import (
	"context"

	"git.perx.ru/perxis/perxis-go/pkg/cache"
	"git.perx.ru/perxis/perxis-go/pkg/data"
	"git.perx.ru/perxis/perxis-go/pkg/options"
	service "git.perx.ru/perxis/perxis-go/pkg/spaces"
)

func orgKey(orgID string) string { return "org-" + orgID }

func CachingMiddleware(cache cache.Cache) Middleware {
	return func(next service.Spaces) service.Spaces {
		m := &cachingMiddleware{
			cache: cache,
			next:  next,
		}

		return m
	}
}

type cachingMiddleware struct {
	cache cache.Cache
	next  service.Spaces
}

func (m cachingMiddleware) Create(ctx context.Context, space *service.Space) (sp *service.Space, err error) {

	sp, err = m.next.Create(ctx, space)
	if err == nil {
		_ = m.cache.Remove(orgKey(sp.OrgID))
	}
	return sp, err
}

func (m cachingMiddleware) Get(ctx context.Context, id string) (sp *service.Space, err error) {

	value, e := m.cache.Get(id)
	if e == nil {
		return value.(*service.Space).Clone(), nil
	}
	sp, err = m.next.Get(ctx, id)
	if err == nil {
		_ = m.cache.Set(id, sp)
		return sp.Clone(), nil
	}
	return nil, err
}

func (m cachingMiddleware) List(ctx context.Context, orgId string) (spaces []*service.Space, err error) {

	value, e := m.cache.Get(orgKey(orgId))
	if e == nil {
		return data.CloneSlice(value.([]*service.Space)), nil
	}
	spaces, err = m.next.List(ctx, orgId)
	if err == nil {
		_ = m.cache.Set(orgKey(orgId), spaces)
		for _, s := range spaces {
			_ = m.cache.Set(s.ID, s)
		}
		return data.CloneSlice(spaces), nil
	}
	return nil, err
}

func (m cachingMiddleware) Find(ctx context.Context, filter *service.Filter, fo *options.FindOptions) (spaces []*service.Space, total int, err error) {
	return m.next.Find(ctx, filter, fo)
}

func (m cachingMiddleware) Update(ctx context.Context, space *service.Space) (err error) {

	err = m.next.Update(ctx, space)
	if err == nil {
		value, e := m.cache.Get(space.ID)
		if e == nil {
			space := value.(*service.Space)
			_ = m.cache.Remove(orgKey(space.OrgID))
		}
		_ = m.cache.Remove(space.ID)
	}
	return err
}

func (m cachingMiddleware) UpdateConfig(ctx context.Context, id string, config *service.Config) (err error) {

	err = m.next.UpdateConfig(ctx, id, config)
	if err == nil {
		value, e := m.cache.Get(id)
		if e == nil {
			space := value.(*service.Space)
			_ = m.cache.Remove(orgKey(space.OrgID))
		}
		_ = m.cache.Remove(id)
	}
	return err
}

func (m cachingMiddleware) SetState(ctx context.Context, id string, state *service.StateInfo) (err error) {

	err = m.next.SetState(ctx, id, state)
	if err == nil {
		value, e := m.cache.Get(id)
		if e == nil {
			space := value.(*service.Space)
			_ = m.cache.Remove(orgKey(space.OrgID))
		}
		_ = m.cache.Remove(id)
	}
	return err
}

func (m cachingMiddleware) Delete(ctx context.Context, id string) (err error) {

	err = m.next.Delete(ctx, id)
	if err == nil {
		value, e := m.cache.Get(id)
		if e == nil {
			space := value.(*service.Space)
			_ = m.cache.Remove(orgKey(space.OrgID))
		}
		_ = m.cache.Remove(id)
	}
	return err
}

func (m cachingMiddleware) Transfer(ctx context.Context, id, transferToOrg string) error {
	err := m.next.Transfer(ctx, id, transferToOrg)
	if err == nil {
		value, e := m.cache.Get(id)
		if e == nil {
			space := value.(*service.Space)
			_ = m.cache.Remove(orgKey(space.OrgID))
			_ = m.cache.Remove(orgKey(space.TransferToOrg))
		}
		_ = m.cache.Remove(id)
		_ = m.cache.Remove(transferToOrg)
	}
	return err
}

func (m cachingMiddleware) AbortTransfer(ctx context.Context, id string) error {
	err := m.next.AbortTransfer(ctx, id)
	if err == nil {
		value, e := m.cache.Get(id)
		if e == nil {
			space := value.(*service.Space)
			_ = m.cache.Remove(orgKey(space.OrgID))
			_ = m.cache.Remove(orgKey(space.TransferToOrg))
		}
		_ = m.cache.Remove(id)
	}
	return err
}

func (m cachingMiddleware) ListTransfers(ctx context.Context, orgID string) (spaces []*service.Space, err error) {
	return m.next.ListTransfers(ctx, orgID)
}

func (m cachingMiddleware) Move(ctx context.Context, id, orgID string) error {
	err := m.next.Move(ctx, id, orgID)
	if err == nil {
		value, e := m.cache.Get(id)
		if e == nil {
			space := value.(*service.Space)
			_ = m.cache.Remove(orgKey(space.OrgID))
			_ = m.cache.Remove(orgKey(space.TransferToOrg))
		}
		_ = m.cache.Remove(id)
		_ = m.cache.Remove(orgID)
	}
	return err
}
