package middleware

import (
	"context"

	pkgId "git.perx.ru/perxis/perxis-go/id"
	"git.perx.ru/perxis/perxis-go/pkg/options"
	"git.perx.ru/perxis/perxis-go/pkg/users"
	logzap "git.perx.ru/perxis/perxis-go/zap"
	"go.uber.org/zap"
)

type loggingMiddleware struct {
	logger *zap.Logger
	next   users.Users
}

func LoggingMiddleware(logger *zap.Logger) Middleware {
	return func(next users.Users) users.Users {
		return &loggingMiddleware{
			next:   next,
			logger: logger.With(logzap.Component("Users")),
		}
	}
}

func (m *loggingMiddleware) Create(ctx context.Context, create *users.User) (user *users.User, err error) {
	logger := m.logger.With(
		logzap.Caller(ctx),
		logzap.Event(users.EventCreate),
	)

	user, err = m.next.Create(ctx, create)
	if err != nil {
		logger.Error("Failed to create", zap.Error(err), logzap.Channels(logzap.Userlog, logzap.Syslog), logzap.Object(user))
		return
	}

	logger.Info("User created", logzap.Channels(logzap.Userlog), logzap.Object(user))

	return user, err
}

func (m *loggingMiddleware) Get(ctx context.Context, id string) (user *users.User, err error) {
	logger := m.logger.With(
		logzap.Caller(ctx),
		logzap.Object(pkgId.NewUserId(id)),
	)

	user, err = m.next.Get(ctx, id)
	if err != nil {
		logger.Error("Failed to get", zap.Error(err))
		return
	}

	return user, err
}

func (m *loggingMiddleware) Find(ctx context.Context, filter *users.Filter, options *options.FindOptions) (found []*users.User, total int, err error) {
	logger := m.logger.With(
		logzap.Caller(ctx),
	)

	found, total, err = m.next.Find(ctx, filter, options)
	if err != nil {
		logger.Error("Failed to find", zap.Error(err))
		return
	}

	return found, total, err
}

func (m *loggingMiddleware) Update(ctx context.Context, update *users.User) (err error) {
	logger := m.logger.With(
		logzap.Caller(ctx),
		logzap.Event(users.EventUpdate),
		logzap.Object(update),
	)

	err = m.next.Update(ctx, update)
	if err != nil {
		logger.Error("Failed to update", zap.Error(err), logzap.Channels(logzap.Userlog, logzap.Syslog))
		return
	}

	logger.Info("User updated", logzap.Channels(logzap.Userlog))

	return err
}

func (m *loggingMiddleware) Delete(ctx context.Context, id string) (err error) {
	logger := m.logger.With(
		logzap.Caller(ctx),
		logzap.Event(users.EventDelete),
		logzap.Object(pkgId.NewUserId(id)),
	)

	err = m.next.Delete(ctx, id)
	if err != nil {
		logger.Error("Failed to delete", zap.Error(err), logzap.Channels(logzap.Userlog, logzap.Syslog))
		return
	}

	logger.Info("User deleted", logzap.Channels(logzap.Userlog))

	return err
}

func (m *loggingMiddleware) GetByIdentity(ctx context.Context, identity string) (user *users.User, err error) {
	logger := m.logger.With(
		logzap.Caller(ctx),
		logzap.Object(pkgId.NewUserId(identity)),
	)

	user, err = m.next.GetByIdentity(ctx, identity)
	if err != nil {
		logger.Error("Failed to get by identity", zap.Error(err))
		return
	}
	return user, err
}
